@@ -7,6 +7,7 @@ import unittest
 import OpenSSL
 import mock
 
+from acme import errors as acme_errors
 from acme import jose
 
 from certbot import account
@@ -44,20 +45,25 @@ class RegisterTest(unittest.TestCase):
     def test_no_tos(self):
         with mock.patch("certbot.client.acme_client.Client") as mock_client:
             mock_client.register().terms_of_service = "http://tos"
-            with mock.patch("certbot.account.report_new_account"):
-                self.tos_cb.return_value = False
-                self.assertRaises(errors.Error, self._call)
+            with mock.patch("certbot.eff.handle_subscription") as mock_handle:
+                with mock.patch("certbot.account.report_new_account"):
+                    self.tos_cb.return_value = False
+                    self.assertRaises(errors.Error, self._call)
+                    self.assertFalse(mock_handle.called)
 
-                self.tos_cb.return_value = True
-                self._call()
+                    self.tos_cb.return_value = True
+                    self._call()
+                    self.assertTrue(mock_handle.called)
 
-                self.tos_cb = None
-                self._call()
+                    self.tos_cb = None
+                    self._call()
+                    self.assertEqual(mock_handle.call_count, 2)
 
     def test_it(self):
         with mock.patch("certbot.client.acme_client.Client"):
             with mock.patch("certbot.account.report_new_account"):
-                self._call()
+                with mock.patch("certbot.eff.handle_subscription"):
+                    self._call()
 
     @mock.patch("certbot.account.report_new_account")
     @mock.patch("certbot.client.display_ops.get_email")
@@ -67,9 +73,11 @@ class RegisterTest(unittest.TestCase):
         msg = "DNS problem: NXDOMAIN looking up MX for example.com"
         mx_err = messages.Error.with_code('invalidContact', detail=msg)
         with mock.patch("certbot.client.acme_client.Client") as mock_client:
-            mock_client().register.side_effect = [mx_err, mock.MagicMock()]
-            self._call()
-            self.assertEqual(mock_get_email.call_count, 1)
+            with mock.patch("certbot.eff.handle_subscription") as mock_handle:
+                mock_client().register.side_effect = [mx_err, mock.MagicMock()]
+                self._call()
+                self.assertEqual(mock_get_email.call_count, 1)
+                self.assertTrue(mock_handle.called)
 
     @mock.patch("certbot.account.report_new_account")
     def test_email_invalid_noninteractive(self, _rep):
@@ -77,8 +85,9 @@ class RegisterTest(unittest.TestCase):
         msg = "DNS problem: NXDOMAIN looking up MX for example.com"
         mx_err = messages.Error.with_code('invalidContact', detail=msg)
         with mock.patch("certbot.client.acme_client.Client") as mock_client:
-            mock_client().register.side_effect = [mx_err, mock.MagicMock()]
-            self.assertRaises(errors.Error, self._call)
+            with mock.patch("certbot.eff.handle_subscription"):
+                mock_client().register.side_effect = [mx_err, mock.MagicMock()]
+                self.assertRaises(errors.Error, self._call)
 
     def test_needs_email(self):
         self.config.email = None
@@ -86,21 +95,25 @@ class RegisterTest(unittest.TestCase):
 
     @mock.patch("certbot.client.logger")
     def test_without_email(self, mock_logger):
-        with mock.patch("certbot.client.acme_client.Client"):
-            with mock.patch("certbot.account.report_new_account"):
-                self.config.email = None
-                self.config.register_unsafely_without_email = True
-                self.config.dry_run = False
-                self._call()
-                mock_logger.warning.assert_called_once_with(mock.ANY)
+        with mock.patch("certbot.eff.handle_subscription") as mock_handle:
+            with mock.patch("certbot.client.acme_client.Client"):
+                with mock.patch("certbot.account.report_new_account"):
+                    self.config.email = None
+                    self.config.register_unsafely_without_email = True
+                    self.config.dry_run = False
+                    self._call()
+                    mock_logger.info.assert_called_once_with(mock.ANY)
+                    self.assertTrue(mock_handle.called)
 
     def test_unsupported_error(self):
         from acme import messages
         msg = "Test"
         mx_err = messages.Error(detail=msg, typ="malformed", title="title")
         with mock.patch("certbot.client.acme_client.Client") as mock_client:
-            mock_client().register.side_effect = [mx_err, mock.MagicMock()]
-            self.assertRaises(messages.Error, self._call)
+            with mock.patch("certbot.eff.handle_subscription") as mock_handle:
+                mock_client().register.side_effect = [mx_err, mock.MagicMock()]
+                self.assertRaises(messages.Error, self._call)
+        self.assertFalse(mock_handle.called)
 
 
 class ClientTestCommon(unittest.TestCase):
@@ -158,7 +171,9 @@ class ClientTest(ClientTestCommon):
         self.acme.fetch_chain.assert_called_once_with(mock.sentinel.certr)
 
     @mock.patch("certbot.client.logger")
-    def test_obtain_certificate_from_csr(self, mock_logger):
+    @test_util.patch_get_utility()
+    def test_obtain_certificate_from_csr(self, unused_mock_get_utility,
+                                         mock_logger):
         self._mock_obtain_certificate()
         test_csr = util.CSR(form="der", file=None, data=CSR_SAN)
         auth_handler = self.client.auth_handler
@@ -191,8 +206,44 @@ class ClientTest(ClientTestCommon):
             test_csr)
         mock_logger.warning.assert_called_once_with(mock.ANY)
 
+    @test_util.patch_get_utility()
+    def test_obtain_certificate_from_csr_retry_succeeded(
+            self, mock_get_utility):
+        self._mock_obtain_certificate()
+        self.acme.fetch_chain.side_effect = [acme_errors.Error,
+                                             mock.sentinel.chain]
+        test_csr = util.CSR(form="der", file=None, data=CSR_SAN)
+        auth_handler = self.client.auth_handler
+
+        authzr = auth_handler.get_authorizations(self.eg_domains, False)
+        self.assertEqual(
+            (mock.sentinel.certr, mock.sentinel.chain),
+            self.client.obtain_certificate_from_csr(
+                self.eg_domains,
+                test_csr,
+                authzr=authzr))
+        self.assertEqual(1, mock_get_utility().notification.call_count)
+
+    @test_util.patch_get_utility()
+    def test_obtain_certificate_from_csr_retry_failed(self, mock_get_utility):
+        self._mock_obtain_certificate()
+        self.acme.fetch_chain.side_effect = acme_errors.Error
+        test_csr = util.CSR(form="der", file=None, data=CSR_SAN)
+        auth_handler = self.client.auth_handler
+
+        authzr = auth_handler.get_authorizations(self.eg_domains, False)
+        self.assertRaises(
+            acme_errors.Error,
+            self.client.obtain_certificate_from_csr,
+            self.eg_domains,
+            test_csr,
+            authzr=authzr)
+        self.assertEqual(1, mock_get_utility().notification.call_count)
+
     @mock.patch("certbot.client.crypto_util")
-    def test_obtain_certificate(self, mock_crypto_util):
+    @test_util.patch_get_utility()
+    def test_obtain_certificate(self, unused_mock_get_utility,
+                                mock_crypto_util):
         self._mock_obtain_certificate()
 
         csr = util.CSR(form="der", file=None, data=CSR_SAN)
