@@ -1,10 +1,12 @@
 """Test :mod:`certbot.display.util`."""
+import inspect
 import os
 import unittest
 
 import mock
 
-import certbot.errors as errors
+from certbot import errors
+from certbot import interfaces
 
 from certbot.display import util as display_util
 
@@ -13,122 +15,6 @@ CHOICES = [("First", "Description1"), ("Second", "Description2")]
 TAGS = ["tag1", "tag2", "tag3"]
 TAGS_CHOICES = [("1", "tag1"), ("2", "tag2"), ("3", "tag3")]
 
-
-class NcursesDisplayTest(unittest.TestCase):
-    """Test ncurses display.
-
-    Since this is mostly a wrapper, it might be more helpful to test the
-    actual dialog boxes. The test file located in ./tests/display.py
-    (relative to the root of the repository) will actually display the
-    various boxes but requires the user to do the verification. If
-    something seems amiss please use that test script to debug it, the
-    automatic tests rely on too much mocking.
-
-    """
-    def setUp(self):
-        super(NcursesDisplayTest, self).setUp()
-        self.displayer = display_util.NcursesDisplay()
-
-        self.default_menu_options = {
-            "choices": CHOICES,
-            "ok_label": "OK",
-            "cancel_label": "Cancel",
-            "help_button": False,
-            "help_label": "",
-            "width": display_util.WIDTH,
-            "height": display_util.HEIGHT,
-            "menu_height": display_util.HEIGHT - 6,
-        }
-
-    @mock.patch("certbot.display.util.dialog.Dialog.msgbox")
-    def test_notification(self, mock_msgbox):
-        """Kind of worthless... one liner."""
-        self.displayer.notification("message")
-        self.assertEqual(mock_msgbox.call_count, 1)
-
-    @mock.patch("certbot.display.util.dialog.Dialog.menu")
-    def test_menu_tag_and_desc(self, mock_menu):
-        mock_menu.return_value = (display_util.OK, "First")
-
-        ret = self.displayer.menu("Message", CHOICES)
-        mock_menu.assert_called_with("Message", **self.default_menu_options)
-
-        self.assertEqual(ret, (display_util.OK, 0))
-
-    @mock.patch("certbot.display.util.dialog.Dialog.menu")
-    def test_menu_tag_and_desc_cancel(self, mock_menu):
-        mock_menu.return_value = (display_util.CANCEL, "")
-
-        ret = self.displayer.menu("Message", CHOICES)
-
-        mock_menu.assert_called_with("Message", **self.default_menu_options)
-
-        self.assertEqual(ret, (display_util.CANCEL, -1))
-
-    @mock.patch("certbot.display.util.dialog.Dialog.menu")
-    def test_menu_desc_only(self, mock_menu):
-        mock_menu.return_value = (display_util.OK, "1")
-
-        ret = self.displayer.menu("Message", TAGS, help_label="More Info")
-
-        self.default_menu_options.update(
-            choices=TAGS_CHOICES, help_button=True, help_label="More Info")
-        mock_menu.assert_called_with("Message", **self.default_menu_options)
-
-        self.assertEqual(ret, (display_util.OK, 0))
-
-    @mock.patch("certbot.display.util.dialog.Dialog.menu")
-    def test_menu_desc_only_help(self, mock_menu):
-        mock_menu.return_value = (display_util.HELP, "2")
-
-        ret = self.displayer.menu("Message", TAGS, help_label="More Info")
-
-        self.assertEqual(ret, (display_util.HELP, 1))
-
-    @mock.patch("certbot.display.util.dialog.Dialog.menu")
-    def test_menu_desc_only_cancel(self, mock_menu):
-        mock_menu.return_value = (display_util.CANCEL, "")
-
-        ret = self.displayer.menu("Message", TAGS, help_label="More Info")
-
-        self.assertEqual(ret, (display_util.CANCEL, -1))
-
-    @mock.patch("certbot.display.util."
-                "dialog.Dialog.inputbox")
-    def test_input(self, mock_input):
-        mock_input.return_value = (mock.MagicMock(), mock.MagicMock())
-        self.displayer.input("message")
-        self.assertEqual(mock_input.call_count, 1)
-
-    @mock.patch("certbot.display.util.dialog.Dialog.yesno")
-    def test_yesno(self, mock_yesno):
-        mock_yesno.return_value = display_util.OK
-
-        self.assertTrue(self.displayer.yesno("message"))
-
-        mock_yesno.assert_called_with(
-            "message", yes_label="Yes", no_label="No")
-
-    @mock.patch("certbot.display.util."
-                "dialog.Dialog.checklist")
-    def test_checklist(self, mock_checklist):
-        mock_checklist.return_value = (mock.MagicMock(), mock.MagicMock())
-        self.displayer.checklist("message", TAGS)
-
-        choices = [
-            (TAGS[0], "", True),
-            (TAGS[1], "", True),
-            (TAGS[2], "", True),
-        ]
-        mock_checklist.assert_called_with("message", choices=choices)
-
-    @mock.patch("certbot.display.util.dialog.Dialog.dselect")
-    def test_directory_select(self, mock_dselect):
-        mock_dselect.return_value = (mock.MagicMock(), mock.MagicMock())
-        self.displayer.directory_select("message")
-        self.assertEqual(mock_dselect.call_count, 1)
-
-
 class FileOutputDisplayTest(unittest.TestCase):
     """Test stdout display.
 
@@ -136,85 +22,149 @@ class FileOutputDisplayTest(unittest.TestCase):
     functions look to a user, uncomment the test_visual function.
 
     """
+    # pylint:disable=too-many-public-methods
     def setUp(self):
         super(FileOutputDisplayTest, self).setUp()
         self.mock_stdout = mock.MagicMock()
-        self.displayer = display_util.FileDisplay(self.mock_stdout)
+        self.displayer = display_util.FileDisplay(self.mock_stdout, False)
 
     def test_notification_no_pause(self):
-        self.displayer.notification("message", 10, False)
+        self.displayer.notification("message", False)
         string = self.mock_stdout.write.call_args[0][0]
 
         self.assertTrue("message" in string)
 
     def test_notification_pause(self):
         with mock.patch("six.moves.input", return_value="enter"):
-            self.displayer.notification("message")
+            self.displayer.notification("message", force_interactive=True)
 
         self.assertTrue("message" in self.mock_stdout.write.call_args[0][0])
 
+    def test_notification_noninteractive(self):
+        self._force_noninteractive(self.displayer.notification, "message")
+        string = self.mock_stdout.write.call_args[0][0]
+        self.assertTrue("message" in string)
+
+    def test_notification_noninteractive2(self):
+        # The main purpose of this test is to make sure we only call
+        # logger.warning once which _force_noninteractive checks internally
+        self._force_noninteractive(self.displayer.notification, "message")
+        string = self.mock_stdout.write.call_args[0][0]
+        self.assertTrue("message" in string)
+
+        self.assertTrue(self.displayer.skipped_interaction)
+
+        self._force_noninteractive(self.displayer.notification, "message2")
+        string = self.mock_stdout.write.call_args[0][0]
+        self.assertTrue("message2" in string)
+
     @mock.patch("certbot.display.util."
                 "FileDisplay._get_valid_int_ans")
     def test_menu(self, mock_ans):
         mock_ans.return_value = (display_util.OK, 1)
-        ret = self.displayer.menu("message", CHOICES)
+        ret = self.displayer.menu("message", CHOICES, force_interactive=True)
         self.assertEqual(ret, (display_util.OK, 0))
 
+    def test_menu_noninteractive(self):
+        default = 0
+        result = self._force_noninteractive(
+            self.displayer.menu, "msg", CHOICES, default=default)
+        self.assertEqual(result, (display_util.OK, default))
+
     def test_input_cancel(self):
         with mock.patch("six.moves.input", return_value="c"):
-            code, _ = self.displayer.input("message")
+            code, _ = self.displayer.input("message", force_interactive=True)
 
         self.assertTrue(code, display_util.CANCEL)
 
     def test_input_normal(self):
         with mock.patch("six.moves.input", return_value="domain.com"):
-            code, input_ = self.displayer.input("message")
+            code, input_ = self.displayer.input("message", force_interactive=True)
 
         self.assertEqual(code, display_util.OK)
         self.assertEqual(input_, "domain.com")
 
+    def test_input_noninteractive(self):
+        default = "foo"
+        code, input_ = self._force_noninteractive(
+            self.displayer.input, "message", default=default)
+
+        self.assertEqual(code, display_util.OK)
+        self.assertEqual(input_, default)
+
+    def test_input_assertion_fail(self):
+        # If the call to util.assert_valid_call is commented out, an
+        # error.Error is raised, otherwise, an AssertionError is raised.
+        self.assertRaises(Exception, self._force_noninteractive,
+                          self.displayer.input, "message", cli_flag="--flag")
+
+    def test_input_assertion_fail2(self):
+        with mock.patch("certbot.display.util.assert_valid_call"):
+            self.assertRaises(errors.Error, self._force_noninteractive,
+                              self.displayer.input, "msg", cli_flag="--flag")
+
     def test_yesno(self):
         with mock.patch("six.moves.input", return_value="Yes"):
-            self.assertTrue(self.displayer.yesno("message"))
+            self.assertTrue(self.displayer.yesno(
+                "message", force_interactive=True))
         with mock.patch("six.moves.input", return_value="y"):
-            self.assertTrue(self.displayer.yesno("message"))
+            self.assertTrue(self.displayer.yesno(
+                "message", force_interactive=True))
         with mock.patch("six.moves.input", side_effect=["maybe", "y"]):
-            self.assertTrue(self.displayer.yesno("message"))
+            self.assertTrue(self.displayer.yesno(
+                "message", force_interactive=True))
         with mock.patch("six.moves.input", return_value="No"):
-            self.assertFalse(self.displayer.yesno("message"))
+            self.assertFalse(self.displayer.yesno(
+                "message", force_interactive=True))
         with mock.patch("six.moves.input", side_effect=["cancel", "n"]):
-            self.assertFalse(self.displayer.yesno("message"))
+            self.assertFalse(self.displayer.yesno(
+                "message", force_interactive=True))
 
         with mock.patch("six.moves.input", return_value="a"):
-            self.assertTrue(self.displayer.yesno("msg", yes_label="Agree"))
+            self.assertTrue(self.displayer.yesno(
+                "msg", yes_label="Agree", force_interactive=True))
 
-    @mock.patch("certbot.display.util.FileDisplay.input")
+    def test_yesno_noninteractive(self):
+        self.assertTrue(self._force_noninteractive(
+            self.displayer.yesno, "message", default=True))
+
+    @mock.patch("certbot.display.util.six.moves.input")
     def test_checklist_valid(self, mock_input):
-        mock_input.return_value = (display_util.OK, "2 1")
-        code, tag_list = self.displayer.checklist("msg", TAGS)
+        mock_input.return_value = "2 1"
+        code, tag_list = self.displayer.checklist(
+            "msg", TAGS, force_interactive=True)
         self.assertEqual(
             (code, set(tag_list)), (display_util.OK, set(["tag1", "tag2"])))
 
-    @mock.patch("certbot.display.util.FileDisplay.input")
-    def test_checklist_miss_valid(self, mock_input):
-        mock_input.side_effect = [
-            (display_util.OK, "10"),
-            (display_util.OK, "tag1 please"),
-            (display_util.OK, "1")
-        ]
+    @mock.patch("certbot.display.util.six.moves.input")
+    def test_checklist_empty(self, mock_input):
+        mock_input.return_value = ""
+        code, tag_list = self.displayer.checklist("msg", TAGS, force_interactive=True)
+        self.assertEqual(
+            (code, set(tag_list)), (display_util.OK, set(["tag1", "tag2", "tag3"])))
 
-        ret = self.displayer.checklist("msg", TAGS)
+    @mock.patch("certbot.display.util.six.moves.input")
+    def test_checklist_miss_valid(self, mock_input):
+        mock_input.side_effect = ["10", "tag1 please", "1"]
+
+        ret = self.displayer.checklist("msg", TAGS, force_interactive=True)
         self.assertEqual(ret, (display_util.OK, ["tag1"]))
 
-    @mock.patch("certbot.display.util.FileDisplay.input")
+    @mock.patch("certbot.display.util.six.moves.input")
     def test_checklist_miss_quit(self, mock_input):
-        mock_input.side_effect = [
-            (display_util.OK, "10"),
-            (display_util.CANCEL, "1")
-        ]
-        ret = self.displayer.checklist("msg", TAGS)
+        mock_input.side_effect = ["10", "c"]
+
+        ret = self.displayer.checklist("msg", TAGS, force_interactive=True)
         self.assertEqual(ret, (display_util.CANCEL, []))
 
+    def test_checklist_noninteractive(self):
+        default = TAGS
+        code, input_ = self._force_noninteractive(
+            self.displayer.checklist, "msg", TAGS, default=default)
+
+        self.assertEqual(code, display_util.OK)
+        self.assertEqual(input_, default)
+
     def test_scrub_checklist_input_valid(self):
         # pylint: disable=protected-access
         indices = [
@@ -232,14 +182,38 @@ class FileOutputDisplayTest(unittest.TestCase):
                 self.displayer._scrub_checklist_input(list_, TAGS))
             self.assertEqual(set_tags, exp[i])
 
-    @mock.patch("certbot.display.util.FileDisplay.input")
+    @mock.patch("certbot.display.util.six.moves.input")
     def test_directory_select(self, mock_input):
-        message = "msg"
-        result = (display_util.OK, "/var/www/html",)
-        mock_input.return_value = result
+        # pylint: disable=star-args
+        args = ["msg", "/var/www/html", "--flag", True]
+        user_input = "/var/www/html"
+        mock_input.return_value = user_input
 
-        self.assertEqual(self.displayer.directory_select(message), result)
-        mock_input.assert_called_once_with(message)
+        returned = self.displayer.directory_select(*args)
+        self.assertEqual(returned, (display_util.OK, user_input))
+
+    def test_directory_select_noninteractive(self):
+        default = "/var/www/html"
+        code, input_ = self._force_noninteractive(
+            self.displayer.directory_select, "msg", default=default)
+
+        self.assertEqual(code, display_util.OK)
+        self.assertEqual(input_, default)
+
+    def _force_noninteractive(self, func, *args, **kwargs):
+        skipped_interaction = self.displayer.skipped_interaction
+
+        with mock.patch("certbot.display.util.sys.stdin") as mock_stdin:
+            mock_stdin.isatty.return_value = False
+            with mock.patch("certbot.display.util.logger") as mock_logger:
+                result = func(*args, **kwargs)
+
+        if skipped_interaction:
+            self.assertFalse(mock_logger.warning.called)
+        else:
+            self.assertEqual(mock_logger.warning.call_count, 1)
+
+        return result
 
     def test_scrub_checklist_input_invalid(self):
         # pylint: disable=protected-access
@@ -294,6 +268,13 @@ class FileOutputDisplayTest(unittest.TestCase):
                     self.displayer._get_valid_int_ans(3),
                     (display_util.CANCEL, -1))
 
+    def test_methods_take_force_interactive(self):
+        # Every IDisplay method implemented by FileDisplay must take
+        # force_interactive to prevent workflow regressions.
+        for name in interfaces.IDisplay.names():  # pylint: disable=no-member
+            arg_spec = inspect.getargspec(getattr(self.displayer, name))
+            self.assertTrue("force_interactive" in arg_spec.args)
+
 
 class NoninteractiveDisplayTest(unittest.TestCase):
     """Test non-interactive display.
@@ -344,6 +325,16 @@ class NoninteractiveDisplayTest(unittest.TestCase):
         self.assertRaises(
             errors.MissingCommandlineFlag, self.displayer.directory_select, "msg")
 
+    def test_methods_take_kwargs(self):
+        # Every IDisplay method implemented by NoninteractiveDisplay
+        # should take **kwargs because every method of FileDisplay must
+        # take force_interactive which doesn't apply to
+        # NoninteractiveDisplay.
+        for name in interfaces.IDisplay.names():  # pylint: disable=no-member
+            method = getattr(self.displayer, name)
+            # asserts method accepts arbitrary keyword arguments
+            self.assertFalse(inspect.getargspec(method).keywords is None)
+
 
 class SeparateListInputTest(unittest.TestCase):
     """Test Module functions."""
